#include <stdio.h>
#include <stdlib.h>

#include <sys/ioctl.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/soundcard.h>

#include <errno.h>

#include "famaudio.h"

/*** For now we will handle only one Conversion - Mono to Stereo 
 *** Since iPAQ sets up two channels always
 ***/ 
static int   convert_mono  = 0;
static short *proxy_buffer = NULL; 

/* Open the audio device with the desired parameters and return the
 * audio device file descriptor
 */
int openAudio(AudioSpec *spec)
{
  int audio_fd;
  int frag_spec;
  int frag, format, channels,  freq;

  /* Calculate Sample size in bytes */
  spec->size       = SAMPLE_SIZE * spec->channels * spec->samples;
  spec->write_size = SAMPLE_SIZE * spec->channels * spec->write_samples;
  
  /* Open Audio Device */
  if ((audio_fd = open(AUDIO_DEVICE_NAME, O_WRONLY)) == -1) {
    perror(AUDIO_DEVICE_NAME);
    return -1;
  }
  
  /* Set various device parameters */
  /*** Should check for the things we actually get back ***/ 

  /* Set fragment size and no. of fragments */
  for ( frag_spec = 0; (0x01<<frag_spec) < spec->size; ++frag_spec );
  if ( (0x01<<frag_spec) != spec->size ) {
    fprintf(stderr, "Fragment size must be a power of 2!\n");
    return(-1);
  }
  
  /*** Review ***/
  frag_spec |= 0x7fff0000; /* Have as many fragments as possible */
  //frag_spec |= 0x00040000; /* Have only 4 fragments */

  if ( ioctl(audio_fd, SNDCTL_DSP_SETFRAGMENT, &frag_spec) < 0 ) {
    perror("SNDCTL_DSP_SETFRAGMENT");
    return -1;	  
  }
  
  /* Set Audio Format */
  format = AFMT_S16_NE;    
  if (ioctl(audio_fd, SNDCTL_DSP_SETFMT, &format) == -1) {
    perror("SNDCTL_DSP_SETFMT");
    return -1;
  }
  
  /* Set the number of channels */
  /* Seem to be able to set only 2 channels on the iPAQ */
  channels = spec->channels;
  //printf("No. of channels requested = %d\n", channels);
  if (ioctl(audio_fd, SNDCTL_DSP_CHANNELS, &channels) == -1) {
    perror("SNDCTL_DSP_CHANNELS");
    return -1;
  }
  //printf("No. of channels got = %d\n", channels);

  /* Be sure not to catch any other condition */
  if ((spec->channels == 1) && (channels == 2)) {
    convert_mono = 1;
    proxy_buffer = (short *)malloc(spec->max_write_samples
				   * spec->channels 
				   * SAMPLE_SIZE 
				   * 2);   
  }
  
  /* Set sample rate */
  freq = spec->freq;
  if (ioctl(audio_fd, SNDCTL_DSP_SPEED, &freq) == -1) {
    perror("SNDCTL_DSP_SPEED");
    return -1;
  }
  /* Save back what we actually got */
  spec->freq = freq;

  return audio_fd;
}

/* Change the write size in samples (NOT audio buffer sample size) 
 * Doesn't really affect physical device parameters
 */
int setAudioWriteSamples(AudioSpec *spec)
{
  /* Recalculate write size */
  spec->write_size = SAMPLE_SIZE * spec->channels * spec->write_samples;
  return 0;
}

/* Write the given audio buffer to the device */
int playAudio(int audio_fd, short *buffer, int length)
{
  int  written, p = 0;

  /* Mono to stero conversion */  
  if (convert_mono) {    
    int i;
    
    /* Copy Mono data to left and right channels */
    /* Any other faster way?? */
    for (i = 0;i < (length/SAMPLE_SIZE);i++) {
      proxy_buffer[2*i]   = buffer[i];
      proxy_buffer[2*i+1] = buffer[i];
    }

    length *= 2;    
  }
  else {
    /* No conversion needed */
    proxy_buffer = buffer;
  }

  do { 
    
    written = write(audio_fd, (void *)&proxy_buffer[p], length - p);
    
    if (written > 0) {
      p += written;
    }
    else {
      if (written == -1 && errno != 0 && errno != EAGAIN && errno != EINTR) {
	/* Non recoverable error has occurred.It should be reported!!! */
	perror("audio");
	return -1;
      }
    }
    
  } while (p < length);

  return 0;
}

/* Close the audio device */
void closeAudio(int audio_fd)
{
  if (convert_mono) {
    free(proxy_buffer);
  }
  
  close(audio_fd);
}


