/*  
 * famuadio.h - Audio Related functions
 */

#define AUDIO_DEVICE_NAME    "/dev/dsp"

/* Our sample size is always 2 bytes */
#define SAMPLE_SIZE 2

typedef struct _audio_spec {
  int  freq;	          /* DSP frequency -- samples per second */
  int  channels;          /* Number of channels: 1 mono, 2 stereo */
  int  samples;	          /* Audio buffer size in samples */
  int  size;	          /* Audio buffer size in bytes (calculated) */  

  int  write_samples;     /* No. of samples written in one write
			   * Corresponds to the number of samples decoded
			   * by the player in one go */
  int  max_write_samples; /* Max value for above */    
  int  write_size;        /* Same as above - in bytes (Calculated) */

} AudioSpec;

/* Open the audio device with the desired parameters and return the
 * audio device file descriptor
 */
int  openAudio(AudioSpec *spec);

/* Change the write size in samples (NOT audio buffer sample size) 
 * Doesn't really affect physical device parameters
 */
int setAudioWriteSamples(AudioSpec *spec);

/* Write the given audio buffer to the device */
int playAudio(int audio_fd, short *buffer, int length);

/* Close the audio device */
void closeAudio(int audio_fd);

/* Sync audio device - Basically reopens the device */
void syncAudio(int *audio_fd, AudioSpec *spec);




