#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include <dlfcn.h>

#include "famdecoderplugin.h"

InitDecoderFunc initDecoder;
DestroyDecoderFunc destroyDecoder;

/* Get the plugin that supports the format */
static FamDecoderPlugin *getSupportingPlugin(char *plugin_path,
					     char *file_path)
{  
  char *full_plugin_path;
  void *handle;
  FamDecoder *decoder;
  FamDecoderPlugin *plugin;

  /* PLUGIN_DIR is picked from the build environment */
  full_plugin_path = (char *)malloc(strlen(PLUGIN_DIR) + 1 
				    + strlen(plugin_path) + 1);
  if (full_plugin_path == NULL)
    return NULL;
  
  strcpy(full_plugin_path, PLUGIN_DIR);
  strcat(full_plugin_path, "/");
  strcat(full_plugin_path, plugin_path);

#ifdef __DEBUG
  printf("Trying plugin : %s\n", full_plugin_path);
#endif

  /* Open the DLL */
  handle = dlopen(full_plugin_path, RTLD_NOW);
  if (handle == NULL) {
    printf("%s\n",dlerror());
    free(full_plugin_path);
    return NULL;    
  }  
  free(full_plugin_path);

  /* Get the Decoder object */
  /* All plugins must export this function */
  initDecoder = (InitDecoderFunc) dlsym(handle, "initDecoder");
  if (initDecoder == NULL) {
    printf("%s\n", dlerror());
    dlclose(handle);
    return NULL;
  }
  

  /* Get the decoder object if the plugin supports the file format */
  decoder = (*initDecoder)(file_path);
  if (decoder == NULL) {    
#ifdef __DEBUG
    printf(" Plugin does not support the file format\n");
#endif
    dlclose(handle);
    return NULL;
  }

  /* Allocate the plugin object an return it */
  plugin = (FamDecoderPlugin *)malloc(sizeof(FamDecoderPlugin));
  if (plugin == NULL) {
    printf("Out of memory?\n");     
    dlclose(handle);
    /*** Some more cleanup reqd. here ***/
    return NULL;
  }

  plugin->handle  = handle;  
  plugin->decoder = decoder;  
  return plugin;
}

/* Dynamically load the required decoder given the mdeia file path 
 * It will search for the plugin that can handle the given media file
 */
FamDecoderPlugin *loadDecoderPlugin(char *file_path)
{
  FamDecoderPlugin *plugin = NULL;
  struct dirent **namelist;
  int n, found = 0;

  /* Search through the plugins directory */
  n = scandir(PLUGIN_DIR, &namelist, 0, alphasort);
  if (n < 0) {
    perror(PLUGIN_DIR);
    return NULL;
  }
  else {
    
    /* Go through each file name */
    while(n--) {

      if (!found) {

	char *plugin_path  = namelist[n]->d_name;
	int len = strlen(plugin_path);
	
	/* Check if the file name ends in ".so" */
	if ((len > 3) && (!strncmp(plugin_path + len - 3, ".so", 3))) {	  

	  if ((plugin = getSupportingPlugin(plugin_path, file_path)) != NULL) {
	    found = 1;
	  }
	  
	}	
      }
      free(namelist[n]);
    }
    free(namelist);
  } 

  return plugin;
}

/* Unload the decoder */
void unloadDecoderPlugin(FamDecoderPlugin *plugin)
{
  destroyDecoder = (DestroyDecoderFunc)dlsym(plugin->handle, "destroyDecoder");
  if (destroyDecoder != NULL) {
    (*destroyDecoder)(plugin->decoder);
  }
  
  dlclose(plugin->handle);
  free(plugin);
}
