/* famdecoder.h - The interface for the Decoder plugins
 * The decoder interface is borrowed from libmpeg3
 */
#ifndef _FAMDECODERPLUGIN_H
#define _FAMDECODERPLUGIN_H

#ifdef __cplusplus
extern "C"
{
#endif /* __cplusplus */

typedef struct _fam_decoder {

  /** COMMON **/
  /* Return a name for the decoder plugin */
  const char *(*decoderName)(void); 

  /* Opens the file and returns a decoder specific handle */
  void *(*openFile)(char *path);
  
  /* Close the decoder specific handle */
  void (*closeHandle)(void *handle);

  /** VIDEO **/
  /* Tells whether a video stream is present */
  int   (*hasVideo)(void *handle);
  
  /* Get video dimensions */
  void  (*getVideoSize)(void *handle, int *width, int *height);
  
  /* Get video frame rate */
  void  (*getVideoFrameRate)(void *handle, float *frame_rate);
  
  /* Get total no. of video frames */
  void  (*getVideoFrames)(void *handle, long *frames);

  /* Decode function - Color model should be RGB888 */
  void  (*readVideoFrame)(void *handle, 
			  unsigned char **output_rows,
			  int in_x,                   
			  int in_y, 
			  int in_w, 
			  int in_h, 
			  int out_w,          
			  int out_h);  

  /** AUDIO **/
  /* Tells whether an audio stream is present */
  int   (*hasAudio)(void *handle);
  
  /* Get Sample rate */
  void  (*getAudioSampleRate)(void *handle, int *sample_rate);
  
  /* Get the number of audio channels - 1 or 2 */
  void  (*getAudioChannels)(void *handle, int *channels);
  
  /* Get total number of samples */
  void  (*getAudioSamples)(void *handle, long *num_samples);
  
  /* Decode function with 16-bit samples */ 
  void  (*readAudioSamples)(void *handle, short *buffer, long num_samples);
  
  /** SEEK **/
  /* Whether the decoder plugin can support frame and sample seeking */
  /* This only means the "set" calls are optional. "get" calls should always
   * be provided
   */
  int (*isSeekable)(void *handle);

  /* Get the current audio position in samples */
  void (*getSample)(void *handle, long *sample);
  
  /* Set the current audio sample position in samples */
  void (*setSample)(void *handle, long sample);
  
  /* Get the current video position in frames */
  void (*getFrame)(void *handle, long *frame);
  
  /* Set the current video poisiton in frames */
  void (*setFrame)(void *handle, long frame);

  
} FamDecoder;


typedef struct _fam_library {
  
  void       *handle;
  FamDecoder *decoder;
  
} FamDecoderPlugin;

/* The decoder init and destroy prototypes */
typedef FamDecoder *(*InitDecoderFunc)(char *file_path);
typedef void (*DestroyDecoderFunc)(FamDecoder *decoder);

/* Dynamically load the required decoder given the mdeia file path 
 * It will search for the plugin that can handle the given media file
 */
FamDecoderPlugin *loadDecoderPlugin(char *file_path);

/* Unload the decoder */
void unloadDecoderPlugin(FamDecoderPlugin *plugin);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
