/* decoder.h - The interface for the Decoder
 * Heavily based on libmpeg3's interface
 */
typdef struct _fam_decoder {

  /** COMMON **/
  /* Says whether the file is compatible with the decoder */
  int   (*checkFileSignature)(char *path);
  
  /* Opens the file and returns a decoder specific handle */
  void *(*openFile)(char *path);
  
  /** VIDEO **/
  /* Tells whether a video stream is present */
  int   (*hasVideo)(void *handle);
  
  /* Get video dimensions */
  void  (*getVideoSize)(void *handle, int *width, int *height);
  
  /* Get video frame rate */
  void  (*getVideoFrameRate)(void *handle, int *frate);
  
  /* Get total no. of video frames */
  void  (*getVideoFrames)(void *handle, long *frames);

  /* Decode function - Color model should be RGB565 */
  void  (*readVideoFrame)(void *handle, 
			  unsigned char **output_rows,
			  int in_x,                   
			  int in_y, 
			  int in_w, 
			  int in_h, 
			  int out_w,          
			  int out_h);  

  /* Skip video frames - For synchronization */
  void  (*skipVideoFrames)(void *handle, int num_frames);
  
  /** AUDIO **/
  /* Tells whether an audio stream is present */
  int   (*hasAudio)(void *handle);
  
  /* Get Sample rate */
  void  (*getAudioSampleRate)(void *handle, int *sample_rate);
  
  /* Get the number of audio channels - 1 or 2 */
  void  (*getAudioChannels)(void *handle, int *channels);
  
  /* Get total number of samples */
  void  (*getAudioSamples)(void *handle, long *num_samples);
  
  /* Decode function with 16-bit samples */ 
  void  (*readAudioSamples)(void *handle, short *buffer, long num_samples);

  /** SEEK **/
  void  (*seekPercentage)(void *handle, double percentage);
  void  (*tellPercentage)(void *handle, double *percentage);
  void  (*getTime)(void *handle, double *time);

} FamDecoder;
